package cp;


public class Result {
  public enum ResultType {NO_RESULT, WHITE_WIN, DRAW, BLACK_WIN, WHITE_WIN_WO, BLACK_WIN_WO, DOUBLE_WO, POSTPONED, CANCELLED};
  private double whiteScore = 0;
  private double blackScore = 0;
  private Game game;
  private ResultType resultType = ResultType.NO_RESULT;

  
  public Result(Game game, double whiteScore, double blackScore, ResultType resultType) {
    this.whiteScore = whiteScore;
    this.blackScore = blackScore;
    this.resultType = resultType;
    this.game = game;
  }
  
  public Result(Game game, ResultType resultType) throws ChessPairerException {
    this.game = game;
    setResultType(resultType);
  }

  public Game getGame() {
    return game;
  }

  public double getWhitePairingScore() {
    return whiteScore;
  }

  public double getBlackPairingScore() {
    return blackScore;
  }

  public double getWhiteScore() {
    if (resultType == ResultType.POSTPONED  && game.getWhitePlayer() != null && game.getBlackPlayer() != null) {
      return 0;
    }
    return whiteScore;
  }

  public double getBlackScore() {
    if (resultType == ResultType.POSTPONED  && game.getWhitePlayer() != null && game.getBlackPlayer() != null) {
      return 0;
    }
    return blackScore;
  }

  public ResultType getResultType() {
    return resultType;
  }
  
  public String toString() {
    if (resultType == ResultType.NO_RESULT) {
      return "";
    }
    else if (resultType == ResultType.POSTPONED) {
      return "uppskj";
    }
    else if (resultType == ResultType.CANCELLED) {
      return "instlld";
    }
    String ans = "";
    int value = (int) Math.floor(whiteScore);
    if (value >= 1 || whiteScore == 0) {
      ans = Integer.toString((int) Math.floor(whiteScore));
    }
    if (whiteScore % 1 == 0.5) {
      ans += "";
    }
    ans += "-";
    value = (int) Math.floor(blackScore);
    if (value >= 1|| blackScore == 0) {
      ans += Integer.toString(value);
    }
    if (blackScore % 1 == 0.5) {
      ans += "";
    }

    if (resultType == ResultType.WHITE_WIN_WO || resultType == ResultType.BLACK_WIN_WO || resultType == ResultType.DOUBLE_WO) {
      ans = ans + " wo";
    }
    return ans;
  }

  public void addWhiteScore(double score) throws ChessPairerException {
    int pointsPerGame = getPointsPerGame();
    if (whiteScore + blackScore + score <= pointsPerGame && score <= getMaxPointsForWin()) {
      if (game.getWhitePlayer() != null) {
        whiteScore += score;
        whiteScore = Math.min(whiteScore, pointsPerGame);
        updateResultType();
      }
    }
    else {
      throw new ChessPairerException("Kan ej tilldela " + (whiteScore + blackScore + score) + " pong till en match.\nFr att ta bort ett resultat, tryck mellanslag.");
    }
  }

  public void addBlackScore(double score) throws ChessPairerException {
    int pointsPerGame = getPointsPerGame();
    if (whiteScore + blackScore + score <= pointsPerGame && score <= getMaxPointsForWin()) {
      if (game.getBlackPlayer() != null) {
        blackScore += score;
        blackScore = Math.min(blackScore, pointsPerGame);
        updateResultType();
      }
    }
    else {
      throw new ChessPairerException("Kan ej tilldela " + (whiteScore + blackScore + score) + " pong till en match.\nFr att ta bort ett resultat, tryck mellanslag.");
    }
  }
  
  private void updateResultType() {
    if (whiteScore > blackScore) {
      resultType = ResultType.WHITE_WIN;
    }
    else if (whiteScore < blackScore) {
      resultType = ResultType.BLACK_WIN;
    }
    else if (whiteScore == blackScore) {
      if (whiteScore == 0) {
        resultType = ResultType.NO_RESULT;
      }
      else {
        resultType = ResultType.DRAW;
      }
    }
  }
  
  public void setResultType(ResultType resultType) throws ChessPairerException {
    this.resultType = resultType;
    int pointsPerGame = getPointsPerGame();
    int maxPointsForWin = getMaxPointsForWin();
    if (resultType == ResultType.WHITE_WIN && game.getWhitePlayer() != null) {
      whiteScore = maxPointsForWin;
      blackScore = pointsPerGame-maxPointsForWin;
    }
		else if (resultType == ResultType.WHITE_WIN_WO && game.getWhitePlayer() != null) {
			whiteScore = maxPointsForWin;
			blackScore = 0;
		}
    else if (resultType == ResultType.DRAW && game.getWhitePlayer() != null && game.getBlackPlayer() != null) {
      whiteScore = pointsPerGame/2.0;
      blackScore = pointsPerGame/2.0;
    }
    else if (resultType == ResultType.BLACK_WIN && game.getBlackPlayer() != null) {
      whiteScore = pointsPerGame-maxPointsForWin;
      blackScore = maxPointsForWin;
    }
		else if (resultType == ResultType.BLACK_WIN_WO && game.getBlackPlayer() != null) {
			whiteScore = 0;
			blackScore = maxPointsForWin;
		}
    else if (resultType == ResultType.POSTPONED  && game.getWhitePlayer() != null && game.getBlackPlayer() != null) {
      whiteScore = maxPointsForWin;
      blackScore = maxPointsForWin;
    }
    else if (resultType == ResultType.DOUBLE_WO) {
      whiteScore = 0;
      blackScore = 0;
    }
    else if (resultType == ResultType.NO_RESULT) {
      whiteScore = 0;
      blackScore = 0;
    }
    else if (resultType == ResultType.CANCELLED) {
      whiteScore = 0;
      blackScore = 0;
    }
    else {
      throw new ChessPairerException("Ej tilltet matchresultat");
    }
  }
  
  public boolean isPlayed() {
    return (resultType == ResultType.WHITE_WIN || resultType == ResultType.BLACK_WIN || resultType == ResultType.DRAW);
  }

  public boolean isToBePlayed() {
    return (resultType == ResultType.NO_RESULT || resultType == ResultType.POSTPONED);
  }

  public static String toString(double score) {
    String ans;
    if (score == 0.5) {
      ans = "";
    }
    else {
      ans = Integer.toString((int) Math.floor(score));
      if (score % 1 == 0.5) {
        ans += "";
      }
    }
    return ans;
  }
  
  double getScore(ResultType resultType) {
    int pointsPerGame = getPointsPerGame();
    int maxPointsForWin = getMaxPointsForWin();
    if (resultType == ResultType.WHITE_WIN || resultType == ResultType.WHITE_WIN_WO || resultType == ResultType.BLACK_WIN || resultType == ResultType.BLACK_WIN_WO || resultType == ResultType.POSTPONED) {
      return maxPointsForWin;
    }
    else if (resultType == ResultType.DRAW) {
      return pointsPerGame/2.0;
    }
    else if (resultType == ResultType.DOUBLE_WO || resultType == ResultType.NO_RESULT || resultType == ResultType.CANCELLED) {
      return 0;
    }
    else {
      System.out.println("Ej tilltet matchresultat.");
      return 0;
    }
  }
	
	public int getPointsPerGame() {
	  return game.getGames().getTournament().getPointsPerGame();
	}
  
  public int getMaxPointsForWin() {
    return game.getGames().getTournament().getMaxPointsForWin();
  }

  public boolean isFinished() {
    return (getPointsPerGame() == blackScore + whiteScore);
  }
}
