package cp.db;

import cp.ChessPairerException;
import cp.Clubs;
import cp.Tournament;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

public class Settings {

  public enum PlayerPresentation {FIRST_LAST, LAST_FIRST};
  private PlayerPresentation playerPresentation;
  private Clubs clubs = new Clubs();
  private boolean maxPointsImmediately = true, searchForUpdate = true;

  public Settings() throws SQLException {
    playerPresentation = loadPlayerPresentation();
    clubs.load();
    maxPointsImmediately = loadMaxPointsImmediately();
    searchForUpdate = loadSearchForUpdate();
  }

  /**
   * Returns the last selected tournament.
   */
  public Tournament getLastTournament() throws SQLException, ChessPairerException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    String sql = "SELECT value FROM settings WHERE setting='lasttournament'";
    ResultSet rs = s.executeQuery(sql);
    if (rs.next()) {
      int index = rs.getInt("value");
      if (index > -1) {
        Tournament tournament = new Tournament(this);
        if (tournament.load(index)) {
          return tournament;
        }
      }
    }
    return null;
  }
  
  /**
   * Sets the last selected tournament.
   */
  public void setLastTournament(Tournament tournament) throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    try {
      String sql = "DELETE FROM settings WHERE setting='lasttournament'";
      s.execute(sql);
      s.close();
      if (tournament != null) {
        sql = "INSERT INTO settings (setting, value) VALUES(?, ?)";
        PreparedStatement s2 = conn.prepareStatement(sql);
        s2.setString(1, "lasttournament");
        s2.setInt(2, tournament.getIndex());
        s2.execute();
        conn.commit();
        s2.close();
      }
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
  }
  
  /**
   * Returns the preferred player presentation.
   */
  public PlayerPresentation getPlayerPresentation() {
    return playerPresentation;
  }

  /**
   * Returns the preferred player presentation.
   */
  private PlayerPresentation loadPlayerPresentation() throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    String sql = "SELECT value FROM settings WHERE setting='playerpresentation'";
    ResultSet rs = s.executeQuery(sql);
    if (rs.next()) {
      int index = rs.getInt("value");
      return PlayerPresentation.values()[index];
    }
    return PlayerPresentation.FIRST_LAST;
  }

  /**
   * Sets the preferred player presentation.
   */
  public void setPlayerPresentation(PlayerPresentation choice) throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    try {
      String sql = "DELETE FROM settings WHERE setting='playerpresentation'";
      s.execute(sql);
      s.close();
      sql = "INSERT INTO settings (setting, value) VALUES(?, ?)";
      PreparedStatement s2 = conn.prepareStatement(sql);
      s2.setString(1, "playerpresentation");
      s2.setInt(2, choice.ordinal());
      s2.execute();
      conn.commit();
      s2.close();
      playerPresentation = choice;
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
  }
  
  /**
   * Returns if max points shall be set immediately when setting results.
   */
  private boolean loadMaxPointsImmediately() throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    String sql = "SELECT value FROM settings WHERE setting='maxpointsimmediately'";
    ResultSet rs = s.executeQuery(sql);
    if (rs.next()) {
      int value = rs.getInt("value");
      return (value == 1 ? true : false);
    }
    return true;
  }

  /**
   * Sets the preferred player presentation.
   */
  public void setMaxPointsImmediately(boolean value) throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    try {
      String sql = "DELETE FROM settings WHERE setting='maxpointsimmediately'";
      s.execute(sql);
      s.close();
      sql = "INSERT INTO settings (setting, value) VALUES(?, ?)";
      PreparedStatement s2 = conn.prepareStatement(sql);
      s2.setString(1, "maxpointsimmediately");
      s2.setInt(2, value ? 1 : 0);
      s2.execute();
      conn.commit();
      s2.close();
      maxPointsImmediately = value;
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
  }

  public boolean getMaxPointsImmediately() {
    return maxPointsImmediately;
  }

  /**
   * Sets the database version.
   */
  public static void setDBVersion(int version) throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    try {
      String sql = "DELETE FROM settings WHERE setting='dbversion'";
      s.execute(sql);
      s.close();
      sql = "INSERT INTO settings (setting, value) VALUES(?,?)";
      PreparedStatement s2 = conn.prepareStatement(sql);
      s2.setString(1, "dbversion");
      s2.setInt(2, version);
      s2.execute();
      conn.commit();
      s2.close();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
  }
  
  /**
   * Returns the preferred player presentation.
   */
  public static int getDBVersion() throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    String sql = "SELECT value FROM settings WHERE setting='dbversion'";
    ResultSet rs = s.executeQuery(sql);
    if (rs.next()) {
      return rs.getInt("value");
    }
    return 0;
  }

  public Clubs getClubs() {
    return clubs;
  }
	
	/**
	 * Returns the nr of rows used when publishing alphabetical pairing.
	 */
	public int getNrOfRows() throws SQLException {
		Connection conn = DB.getConnection();
		Statement s = conn.createStatement();
		String sql = "SELECT value FROM settings WHERE setting='nrofrows'";
		ResultSet rs = s.executeQuery(sql);
		if (rs.next()) {
			return rs.getInt("value");
		}
		return Integer.MAX_VALUE;
	}
	
	/**
	 * Sets the the nr of rows used when publishing alphabetical pairing.
	 */
	public static void setNrOfRows(int rows) throws SQLException {
		Connection conn = DB.getConnection();
		Statement s = conn.createStatement();
		try {
			String sql = "DELETE FROM settings WHERE setting='nrofrows'";
			s.execute(sql);
			s.close();
			sql = "INSERT INTO settings (setting, value) VALUES(?,?)";
			PreparedStatement s2 = conn.prepareStatement(sql);
			s2.setString(1, "nrofrows");
			s2.setInt(2, rows);
			s2.execute();
			conn.commit();
			s2.close();
		}
		catch (SQLException e) {
			conn.rollback();
			throw e;
		}
	}
	
  /**
   * Gets update location.
   */
  public String getUpdateLocation() throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    String sql = "SELECT value FROM stringsettings WHERE setting='updatelocation'";
    ResultSet rs = s.executeQuery(sql);
    if (rs.next()) {
      return rs.getString("value");
    }
    return "https://lass.no-ip.com/Lotta/Lotta/lotta.xml";
  }

  /**
   * Sets update location.
   */
  public static void setUpdateLocation(String location) throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    try {
      String sql = "DELETE FROM stringsettings WHERE setting='updatelocation'";
      s.execute(sql);
      s.close();
      sql = "INSERT INTO stringsettings (setting, value) VALUES(?, ?)";
      PreparedStatement s2 = conn.prepareStatement(sql);
      s2.setString(1, "updatelocation");
      s2.setString(2, location);
      s2.execute();
      conn.commit();
      s2.close();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
  }

  /**
   * Returns if the program shall search for updates.
   */
  private boolean loadSearchForUpdate() throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    String sql = "SELECT value FROM settings WHERE setting='searchforupdate'";
    ResultSet rs = s.executeQuery(sql);
    if (rs.next()) {
      int value = rs.getInt("value");
      return (value == 1 ? true : false);
    }
    return true;
  }

  /**
   * Sets  if the program shall search for updates.
   */
  public void setSearchForUpdate(boolean value) throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    try {
      String sql = "DELETE FROM settings WHERE setting='searchforupdate'";
      s.execute(sql);
      s.close();
      sql = "INSERT INTO settings (setting, value) VALUES(?, ?)";
      PreparedStatement s2 = conn.prepareStatement(sql);
      s2.setString(1, "searchforupdate");
      s2.setInt(2, value ? 1 : 0);
      s2.execute();
      conn.commit();
      s2.close();
      searchForUpdate = value;
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
  }

  public boolean getSearchForUpdate() {
    return searchForUpdate;
  }

}
