package cp.gui.web;

import cp.*;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.filechooser.FileNameExtensionFilter;
import cp.gui.StandingsComparator;
import java.text.SimpleDateFormat;
import java.util.*;


public class FIDEReport {
  private JFrame parent;

  public FIDEReport(JFrame parent) {
    this.parent = parent;
  }

  public void create(Tournament tournament) throws IOException, ChessPairerException {
    if (!tournament.isFinished()) {
      throw new ChessPairerException("Turneringen r inte avslutad n, eller s finns inte alla resultat!");
    }
    JFileChooser fc = new JFileChooser();
    FileNameExtensionFilter filter = new FileNameExtensionFilter("FIDE exchange format", "txt");
    fc.setFileFilter(filter);
    if (fc.showSaveDialog(parent) == JFileChooser.APPROVE_OPTION) {
      File file = fc.getSelectedFile();
      PrintWriter writer;
      writer = new PrintWriter(new FileWriter(file));
      try {
        writeTournamentSection(tournament, writer);
        writePlayerSection(tournament, writer);
      }
      catch (ChessPairerException e) {
        writer.close();
        throw e;
      }
      writer.close();
    }
  }

  public void writeTournamentSection(Tournament tournament, PrintWriter writer) throws ChessPairerException {
    if (tournament.getCity() == null || tournament.getCity().isEmpty() || tournament.getFederation() == null || tournament.getFederation().isEmpty() || tournament.getStartDate() == null 
        || tournament.getEndDate() == null || tournament.getChiefArbiter() == null || tournament.getChiefArbiter().isEmpty() || tournament.getTimePerGame() == null || tournament.getTimePerGame().isEmpty()) {
      throw new ChessPairerException("Uppgifter saknas! Fyll i stad, federation, start/slutdatum, domare och tidskontroll i FIDE-uppgifterna (i turneringsinstllningar).");
    }
    SimpleDateFormat dateFormat = new SimpleDateFormat("yy/MM/dd");
    writer.println(String.format("012 %s", toEnglish(tournament.toString())));
    writer.println(String.format("022 %s", toEnglish(tournament.getCity())));
    writer.println(String.format("032 %s", tournament.getFederation()));
    writer.println(String.format("042 %s", dateFormat.format(tournament.getStartDate())));
    writer.println(String.format("052 %s", dateFormat.format(tournament.getEndDate())));
    writer.println(String.format("062 %s", tournament.getPlayers().size()));
    writer.println(String.format("072 %s", tournament.getPlayers().hasELO()));
    writer.println(String.format("092 %s", tournament.getTypeOfTournament()));
    writer.println(String.format("102 %s", toEnglish(tournament.getChiefArbiter())));
    writer.println(String.format("112 %s", toEnglish(tournament.getDeputyArbiter())));
    writer.println(String.format("122 %s", tournament.getTimePerGame()));
    Vector<RoundDate> roundDates = tournament.getRoundDates();
    if (roundDates.size() > 0) {
      String res = "";
      for (int i=0; i<roundDates.size(); i++) {
        res = res + dateFormat.format(roundDates.get(i).getDate()) + "  ";
      }
      writer.println(String.format("132                                                                                        %s\n", res.trim()));
    }
    int count = 91 + tournament.getNrOfRounds() * 8 + (tournament.getNrOfRounds() - 1) * 2;
    for (int i = 1; i <= count; i++)
      writer.print(i % 10);
    writer.println();
    writer.print("DDD SSSS sTTT NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN RRRR FFF IIIIIIIIIII BBBB/BB/BB PPPP RRRR");
    for (int i = 1; i <= tournament.getNrOfRounds(); i++) {
      writer.print("  " + i % 10 + "" + i % 10 + "" + i % 10 + "" + i % 10 + " " + i % 10 + " " + i % 10);
    }
    writer.println();
    writer.println();
  }

  public void writePlayerSection(Tournament tournament, PrintWriter writer) throws ChessPairerException {
    Players players = (Players) tournament.getPlayers().clone();
    Players order = (Players) tournament.getPlayers().clone();
    Rounds rounds = tournament.getRounds();
    Round lastRound = rounds.get(rounds.size() - 1);
    //Sort players according to result
    StandingsComparator standingsComparator = new StandingsComparator(lastRound);
    Collections.sort(order, standingsComparator);

    int count = 0;
    Iterator<Player> itps = players.iterator();
    while (itps.hasNext()) {
      count++;
      Player player = itps.next();
      String sex = " ";
      if (player.getSex() != null && player.getSex().equals("K")) {
        sex = "w";
      }
      int rank = order.indexOf(player) + 1;
      if (player.getBirthdate() == null) {
        throw new ChessPairerException(String.format("Spelare '%s' saknar fdelsedatum!", player));
      }
      writer.format(Locale.ENGLISH, "001 %1$4d %2$1s%3$3s %4$-33s %5$4d %6$3S %7$11s %8$tY/%8$tm/%8$td %9$4.1f %10$4d", count, sex, player.getTitle(), toEnglish(player.getLastName()) + ", " + toEnglish(player.getFirstName()), player.getRating(),
                    player.getFederation().isEmpty() ? "SWE" : player.getFederation(), player.getFideID(), player.getBirthdate(), player.getScore(lastRound), rank);
      Player opponent = null;
      char color = ' ';
      int i = tournament.getNrOfRounds() - tournament.getNrOfRatedRounds();   
      // Write empty strings for unrated rounds
      for (int j=0; j<i; j++) {
        writer.format("  %1$4s %2$1s %3$1s", "", "", "");        
      }
      for (; i < rounds.size(); i++) {
        Round round = rounds.get(i);
        Game currentGame = round.getGames().getGame(player);
        int opp = 0;
        char res = ' ';
        if (currentGame != null) {
          if (player.equals(currentGame.getWhitePlayer())) {
            opponent = currentGame.getBlackPlayer();
            color = 'w';
          }
          else if (player.equals(currentGame.getBlackPlayer())) {
            opponent = currentGame.getWhitePlayer();
            color = 'b';
          }
          opp = players.indexOf(opponent) + 1;
          switch (currentGame.getResult().getResultType()) {
            case WHITE_WIN:
              if (color == 'b')
                res = '0';
              else
                res = '1';
              break;
            case DRAW:
              res = '=';
              break;
            case BLACK_WIN:
              if (color == 'w')
                res = '0';
              else
                res = '1';
              break;
            case WHITE_WIN_WO:
              if (color == 'b')
                res = '-';
              else
                res = '+';
              break;
            case BLACK_WIN_WO:
              if (color == 'w')
                res = '-';
              else
                res = '+';
              break;
            case DOUBLE_WO:
              res = '-';
              break;
            default:
              break;
          }
        }
        if (opp != 0)
          writer.format("  %1$4d %2$1s %3$1s", opp, color, res);
        else {
          if (currentGame != null) {
            if (currentGame.getResult().getWhiteScore() != 1) {
              res = '=';
            }
            writer.format("  %1$04d %2$1s %3$1s", opp, color, res);
          }
          else {
            // The player was not paired this round.
            writer.format("  %1$4s %2$1s %3$1s", "", "", "");
          }
        }
      }
      writer.println();
    }
  }

  private String toEnglish(String string) {
    string = string.replace('', 'a');
    string = string.replace('', 'a');
    return string.replace('', 'o');
  }
}
